package phyApps.common;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.TextField;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

@SuppressWarnings("serial")
public class ExpandableInputArea extends Panel {
	
	private int numCols;
	private List<InputRow> rows;
	
	public ExpandableInputArea(Panel parentPanel, int cols) {
		numCols = cols;
		rows = new LinkedList<InputRow>();
		
		// Setup the layout.
		setLayout(new GridLayout(0, numCols));
		setBackground(Color.LIGHT_GRAY);
		
		parentPanel.add(this);
	}
	
	public void add(InputRow row) {
		rows.add(row);
		if (row.getRowComponents().size() != numCols)
			throw new IndexOutOfBoundsException("All input rows must be of the same length.");
		
		for (Component component : row)
			add(component);
		setMaximumSize(getPreferredSize());
		setSize(getPreferredSize());
		validate();
	}
	
	public void remove(InputRow row) {
		rows.remove(row);
		for (Component component : row)
			remove(component);
		setSize(getPreferredSize());
		validate();
	}
	
	public Collection<InputRow> getRows() {
		return rows;
	}
	
	/*
	 * Safely read an integer from a text field. Defaults to 0.
	 */
	public static int readInt(TextField field) {
		int result;
		try {
			result = Integer.parseInt(field.getText());
		}
		catch (NumberFormatException e) {
			result = 0;
		}
		return result;
	}
	
	/*
	 * Safely read a double from a text field. Defaults to 0.0.
	 */
	public static double readDouble(TextField field) {
		double result;
		try {
			result = Double.parseDouble(field.getText());
		}
		catch (NumberFormatException e) {
			result = 0.0;
		}
		return result;
	}
}
