package phyApps.common;

import java.awt.Button;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentListener;

@SuppressWarnings("serial")
public abstract class InputArea extends Panel {

	/**
	 * Add a button that tells the 'listener' when it is pressed.
	 * It will give 'command' as the event's action command.
	 * @param button - the button to add
	 * @param command - actionEvent.getActionCommand() will return this string
	 * @param listener - listener.actionPerformed() will be called when this button is pressed.
	 */
	abstract public void add(Button button, String command, ActionListener listener);
	
	/**
	 * Add a scrollbar with no name.
	 * Whenever the scrollbar is moved, an AdjustmentEvent will be sent to listener.
	 * Whenever the scrollbar's value is edited by hand, an Adjustment event will be sent to listener.
	 */
	public void add(Scrollbar scrollbar, AdjustmentListener listener) {
		add(scrollbar, listener, null);
	}
	
	/**
	 * Add a scrollbar with the given name.
	 * The name will be shown on a label on top of the scrollbar.
	 * Whenever the scrollbar is moved, an AdjustmentEvent will be sent to listener.
	 * Whenever the scrollbar's value is edited by hand, an Adjustment event will be sent to listener.
	 * @param name - a string describing the scroll bar.
	 * @param scrollbar - the scrollbar to be added.
	 */
	abstract public void add(Scrollbar scrollbar, AdjustmentListener listener, String name);
	
	/**
	 * Add a text field with no name.
	 */
	public void add(TextField field) {
		add(field, null);
	}
	
	/**
	 * Add a text field with the given name.
	 * The label will be shown to the left of the field.
	 */
	abstract public void add(TextField field, String name);
	
	/**
	 * Add a label.
	 */
	public void add(Label label) {
		add(label, null);
	}
	
	/**
	 * Add a label with a name.
	 */
	abstract public void add(Label label, String name);
}
