package phyApps.common;

import java.awt.Component;
import java.awt.Frame;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

public abstract class Problem {
	
	private static String RESULT_TITLE = "Results";
	private static String CORRECT_MESSAGE = "Correct!";
	private static String INCORRECT_MESSAGE = "Incorrect";
	private static String[] INCORRECT_OPTIONS = {"Retry", "Cancel"};
	protected static double MARGIN_OF_ERROR = 0.05;
	
	abstract public void setup();
	
	
	// DISPLAY A PROBLEM ONCE AND GET THE RESULT:
	
	// Show the problem to the user and check their answer.
	abstract protected Result display(Component parent);
	
	// Display a problem and get the user's answer.
	protected double popup(Component parent, String text, String title) {
		String response = (String) JOptionPane.showInputDialog(parent, text, title,
				JOptionPane.PLAIN_MESSAGE, null, null, "");
		try {
			if (response == null)
				return Double.NaN;
			else
				return Double.valueOf(response);
		}
		catch (NumberFormatException e) {
			return 0.0;
		}
	}
	
	// Display a problem with drop-down options.
	protected String popup(Component parent, String text, String title, String[] options) {
		return (String) JOptionPane.showInputDialog(parent, text, title,
				JOptionPane.PLAIN_MESSAGE, null, options, options[0]);
	}
	
	
	// SHOWING A PROBLEM AND ALLOWING RETRIES:
	
	// Show a problem to the user.
	// If they get it wrong, allow them to retry.
	public boolean show(Component parent) {
		Result result = display(parent);
		if (result == Result.CANCEL)
			return false;
		else if (result == Result.CORRECT) {
			displayCorrectDialog(parent);
			return true;
		}
		else if (result == Result.INCORRECT) {
			boolean retry = displayIncorrectDialog(parent);
			if (retry)
				return show(parent);
			else
				return false;
		}
		return false;
	}

	private void displayCorrectDialog(Component parent) {
		JOptionPane.showMessageDialog(parent, CORRECT_MESSAGE, RESULT_TITLE, JOptionPane.PLAIN_MESSAGE);
	}
	
	private boolean displayIncorrectDialog(Component parent) {
		int answer = JOptionPane.showOptionDialog(parent, INCORRECT_MESSAGE, RESULT_TITLE,
				JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null,
				INCORRECT_OPTIONS, INCORRECT_OPTIONS[0]);
		return answer == 0;
	}
	
	
	// HELPER FUNCTIONS:
	
	protected int randInt(int min, int max, int inc) {
		return min + inc*((int) (((max - min)/inc + 1)*Math.random()));
	}
	
	protected double randDouble(double min, double max, double inc) {
		return min + inc*((int) (((max - min)/inc + 1)*Math.random()));
	}
	
	protected Result checkAnswer(double answer, double correct) {
		if (Double.isNaN(answer))
			return Result.CANCEL;
		double error = (correct - answer)/correct;
		if (error > -MARGIN_OF_ERROR && error < MARGIN_OF_ERROR)
			return Result.CORRECT;
		else
			return Result.INCORRECT;
	}
	
	// A helper function for displaying a new frame with html formatting.
	public static Frame popupFrame(String title, String text) {
		JFrame frame = new JFrame(title);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);		
		frame.add(new JLabel(text));
		frame.pack();
		frame.setVisible(true);
		return frame;
	}
}
