package phyApps.common.graph;

import java.awt.Color;
import java.awt.Graphics;

public class ArrowPlot extends Plot {

	private Point start;
	private Point end;
	private Color color;
	public boolean DRAW_ARROW_HEAD = true;
	public double ARROW_HEAD_FRACTION = .20; // The default size of the arrow head, as a fraction of the arrow's length. 
	public int ARROW_HEAD_MAX = 10; // The maximum size of the arrow head, in pixels.
	public double thickness;
	private static double SKEW = 0.75;
	
	// for the user:
	
	public ArrowPlot(Graph graph, Color color) {
		this(graph, null, null, color);
	}
	
	public ArrowPlot(Graph graph, Color color, Region region) {
		this(graph, null, null, color, region);
	}
	
	public ArrowPlot(Graph graph, Point start, Point end, Color color) {
		super(graph);
		this.start = start;
		this.end = end;
		this.color = color;
		this.thickness = 2.0;
	}
	
	public ArrowPlot(Graph graph, Point start, Point end, Color color, Region region) {
		this(graph, start, end, color);
		setRegion(region);
	}
	
	public void move(Point newStart, Point newEnd) {
		this.start = newStart;
		this.end = newEnd;
	}
	
	public void clear() {
		move(null, null);
	}
	
	// for graph:
	
	public void draw(Graphics g) {
		if (isEmpty())
			return;
		graph.drawLine(g, start, end, color, thickness);
		
		if (!DRAW_ARROW_HEAD)
			return;
		CoordinateSystem space = graph.getCoordinateSpace();
		int length = space.screenDistance(start, end);
		if (length == 0)
			return;
		int size = arrowSize(length);
		int sx = space.getX(start);
		int sy = space.getY(start);
		int ex = space.getX(end);
		int ey = space.getY(end);
		
		int ax = (int) (ex + size*(sx - ex)/length + SKEW*size*(ey - sy)/length);
		int ay = (int) (ey + size*(sy - ey)/length + SKEW*size*(sx - ex)/length);
		int bx = (int) (ex + size*(sx - ex)/length + SKEW*size*(sy - ey)/length);
		int by = (int) (ey + size*(sy - ey)/length + SKEW*size*(ex - sx)/length);
		
		g.setColor(color);
		graph.drawLine(g, space.makePoint(ex, ey), space.makePoint(ax, ay), color, thickness);
		graph.drawLine(g, space.makePoint(ex, ey), space.makePoint(bx, by), color, thickness);
	}
	
	public Region determineRegion() {
		if (isEmpty())
			return null;
		else
			return new Region(start.getX(), start.getY(), end.getX(), end.getY());
	}
	
	// private
	
	public boolean isEmpty() {
		return start == null || end == null;
	}
	
	private int arrowSize(double length) {
		return (int) Math.round(Math.min(ARROW_HEAD_FRACTION*length, ARROW_HEAD_MAX));
	}
}
