package phyApps.common.graph;

import java.awt.Graphics;


public abstract class Plot {
	
	protected Graph graph;
	protected Region region;
	protected boolean auto = true;
	
	// For the user:
	
	public Plot(Graph graph) {
		graph.add(this);
	}
	
	/**
	 * Set the region that the plot takes up.
	 * If region==null, allow the plot to go off screen.
	 */
	public void setRegion(Region region) {
		this.auto = false;
		this.region = region;
		graph.adjust();
	}
	
	/**
	 * Let the plot determine the region it takes up.
	 */
	public void autoRegion() {
		this.auto = true;
		graph.adjust();
	}
	
	// For the Graph:
	
	public void setup(Graph graph) {
		this.graph = graph;
	}
	
	public Region getRegion() {
		if (auto)
			return determineRegion();
		else
			return region;
	}
	
	// Must be implemented:
	
	abstract public boolean isEmpty();
	
	// Automatically determine the region that the plot takes up.
	// null represents an empty region.
	abstract public Region determineRegion();
	
	abstract public void draw(Graphics g);
}
