package phyApps.common.graph;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;



/*
 * A bunch of unconnected points.
 */
public class ScatterPlot extends Plot {

	protected Color color;
	protected List<Point> points;
	protected static int RADIUS = 3;
	
	// for the user:
	
	public void setPoints(List<Point> points) {
		this.points = points;
	}
	
	public void add(Point point) {
		points.add(new Point(point));
	}
	
	public void clear() {
		points = new ArrayList<Point>();
	}
	
	// for the subclasses:
	
	protected ScatterPlot(Graph graph, Color color) {
		super(graph);
		this.points = new CopyOnWriteArrayList<Point>();
		this.color = color;
	}

	// for Graph:
	
	public boolean isEmpty() {
		return points.isEmpty();
	}
	
	public Region determineRegion() {
		double left = 0;
		double right = 0;
		double bottom = 0;
		double top = 0;
		
		for (Point point : points) {
			if (point.getX() < left)
				left = point.getX();
			if (point.getX() > right)
				right = point.getX();
			if (point.getY() < bottom)
				bottom = point.getY();
			if (point.getY() > top)
				top = point.getY();
		}
		
		return new Region(left, bottom, right, top);
	}

	public void draw(Graphics g) {
		for (Point point : points) {
			graph.drawPoint(g, point, color, RADIUS);
		}
	}
}