package phyApps.common.mvc;

import java.awt.Component;
import java.awt.Panel;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * The controller receives input and makes decisions.
 * It knows about both the View and the Model.
 * @author JLP
 *
 */
@SuppressWarnings("serial")
public abstract class Controller extends Panel implements MouseListener, KeyListener {
	
	/**
	 * The controller may cast the View and Model to their subclasses.
	 */
	abstract public void setup(View view, Model model);
	
	/**
	 * Get the controller's panel. This will be the panel shown on the screen.
	 */
	abstract public Component getComponent();
	
	/**
	 * The model may call this when it has updated.
	 * This is helpful for when a control panel needs to show output.
	 */
	abstract public void update();
	
	// The controller can override these methods to use mouse input.
	public void mousePressed(MouseEvent arg0) {}
	public void mouseClicked(MouseEvent arg0) {}
	public void mouseReleased(MouseEvent event) {}
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mouseMoved(MouseEvent arg0) {}
	
	// keyboard input
	public void keyPressed(KeyEvent arg0) {}
	public void keyReleased(KeyEvent arg0) {}
	public void keyTyped(KeyEvent arg0) {}
}
