package phyApps.common.mvc;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Panel;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * To make a physics applet, make subclasses of AppController, AppModel, and AppView.
 * Then subclass the PhyApplet class. Have getController() return a new controller. The same goes for view and model.
 * getTitle() determines the window title.
 * @author JLP
 *
 */

@SuppressWarnings("serial")
public abstract class PhyApplet extends Applet implements WindowListener {
	
	public void setupMVC(Model model, View view, Controller controller) {
		model.setup(view, controller);
		controller.setup(view, model);
		view.draw(model);
		
		layoutApplet(this, controller.getComponent(), view.getComponent());
		
		setSize(getPreferredSize());
	}
	
	public static void layoutApplet(Panel panel, Component controller, Component view) {
		panel.setLayout(new GridBagLayout());
		panel.setBackground(Color.GRAY);
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.FIRST_LINE_START;
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 0.0;
		constraints.weighty = 0.0;
		panel.add(controller, constraints);
		
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;
		constraints.gridx = 1;
		constraints.gridy = 0;
		constraints.weightx = 1.0;
		constraints.weighty = 1.0;
		panel.add(view, constraints);
	}
	
	public static void runApplet(PhyApplet applet, String title) {
		applet.init();
		Frame window = new Frame(title);
		window.add(applet);
		window.addWindowListener(applet);
		window.pack();
		window.setVisible(true);
	}
	
	public void windowActivated(WindowEvent arg0) {}
	public void windowClosed(WindowEvent arg0) {}
	public void windowClosing(WindowEvent arg0) { System.exit(0); }
	public void windowDeactivated(WindowEvent arg0) {}
	public void windowDeiconified(WindowEvent arg0) {}
	public void windowIconified(WindowEvent arg0) {}
	public void windowOpened(WindowEvent arg0) {}
}
