package phyApps.projectile;


import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;

import javax.swing.BoxLayout;

import phyApps.common.ExpandableInputArea;
import phyApps.common.InputRow;
import phyApps.common.graph.AxisPlot;
import phyApps.common.graph.Graph;
import phyApps.common.graph.Point;
import phyApps.common.graph.PointPlot;
import phyApps.common.mvc.PhyApplet;



@SuppressWarnings("serial")
public class ProjectileApplet extends PhyApplet implements WindowListener {
	
	@Override
	public void init() {
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		ProjectileView view = new ProjectileView();
		Panel controller = new ProjectileController(view.getGraph());
		
		add(controller);
		add(view);
		
		setSize(getPreferredSize());
	}
	
	public static void main(String[] args) {
		PhyApplet.runApplet(new ProjectileApplet(), "Projectile");
	}
}


@SuppressWarnings("serial")
class ProjectileView extends Panel {

	private Graph graph;
	
	public ProjectileView() {
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		graph = new Graph(new Dimension(500, 500), this);
		AxisPlot plot = new AxisPlot(graph);
		plot.setXUnit("m");
		plot.setYUnit("m");
		plot.toggleScale(true);
		plot.toggleNumbers(true);
	}
	
	public Graph getGraph() {
		return graph;
	}
}


@SuppressWarnings("serial")
class ProjectileController extends Panel implements ActionListener {
	
	private Graph graph;
	private ExpandableInputArea inputArea;
	
	public ProjectileController(Graph graph) {
		this.graph = graph;
		inputArea = new ExpandableInputArea(this, 7);
		setBackground(Color.GRAY);
		
		// Add a row of labels describing the table.
		InputRow headers = new InputRow();
		String headerNames[] = {"Initial y (m)", "x Velocity (m/s)", "y Velocity (m/s)", "Distance", "Max Height", ""};
		for (int i=0; i<headerNames.length; i++)
			headers.add(new Label(headerNames[i]));
		headers.add(new Button("Add Trial"), "add", this);
		inputArea.add(headers);
		
		// Add a trial.
		inputArea.add(new ProjectileInputRow(graph, this));
	}
	
	public void actionPerformed(ActionEvent event) {
		InputRow row = (InputRow) event.getSource();
		if (event.getActionCommand().equals("add"))
			inputArea.add(new ProjectileInputRow(graph, this));
		else if (event.getActionCommand().equals("remove"))
			inputArea.remove(row);
	}
}


@SuppressWarnings("serial")
class ProjectileInputRow extends InputRow implements ActionListener {

	private ActionListener listener;
	private Graph graph;
	
	private TextField yposField;
	private TextField xvelField;
	private TextField yvelField;
	private Label distanceLabel;
	private Label heightLabel;
	
	// Keep track of the projectile so we can remove it when necessary.
	private Projectile projectile;
	
	public ProjectileInputRow(Graph graph, ActionListener listener) {
		this.listener = listener;
		this.graph = graph;
		
		yposField = new TextField();
		xvelField = new TextField();
		yvelField = new TextField();
		distanceLabel = new Label();
		heightLabel = new Label();
		
		add(yposField);
		add(xvelField);
		add(yvelField);
		add(distanceLabel);
		add(heightLabel);
		add(new Button("Launch"), "run", this);
		add(new Button("Remove"), "remove", this);
	}
	
	private double getYPos() {
		return ExpandableInputArea.readDouble(yposField);
	}
	
	private double getXVel() {
		return ExpandableInputArea.readDouble(xvelField);
	}
	
	private double getYVel() {
		return ExpandableInputArea.readDouble(yvelField);
	}
	
	public void setDistance(double dist) {
		distanceLabel.setText(String.format("%.2f m", dist));
	}
	
	public void setMaxHeight(double height) {
		heightLabel.setText(String.format("%.2f m", height));
	}
	
	public void run() {
		if (projectile != null)
			projectile.remove();
		projectile = new Projectile(graph,
				new Point(0, getYPos()),
				new PointPlot(graph, Color.RED),
				new Point(getXVel(), getYVel()));
		setMaxHeight(projectile.getMaxHeight());
		setDistance(projectile.getDistance());
		projectile.start();
	}
	
	public void remove() {
		if (projectile != null)
			projectile.remove();
	}
	
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("run"))
			run();
		else if (event.getActionCommand().equals("remove"))
			remove();
		event.setSource(this);
		listener.actionPerformed(event);
	}
}

