/*
GNU Lesser General Public License

EkitApplet - Java Swing HTML Editor & Viewer Applet
Copyright (C) 2000 Howard Kistler

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package WebBuilder;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.hexidec.ekit.EkitCore;
import com.hexidec.ekit.EkitCoreSpell;

/** EkitApplet
  * Applet for editing and saving HTML in a Java browser component
  *
  * @author Howard Kistler
  * @version 1.1
  *
  * REQUIREMENTS
  * Java 2 (JDK 1.3 or 1.4)
  * Swing Library
  */

public class HTMLEditor extends JPanel
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -6471515881558733631L;
	/* Components */
	EkitCore ekitCore;

	/** Constructor
	  */
	public HTMLEditor()
	{
		//getRootPane().putClientProperty("defeatSystemEventQueueCheck", Boolean.TRUE);
		init();
	}

	/** Applet Init
	  */
	public void init()
	{
		
		
		
		//String sRawDocument = this.getParameter("DOCUMENT");
		//String sStyleSheetRef = this.getParameter("STYLESHEET");
		String sRawDocument = null;
		String sStyleSheetRef = null;
		//boolean base64 = ((this.getParameter("BASE64") != null) && this.getParameter("BASE64").equalsIgnoreCase("true"));
		boolean base64 = false;
		URL urlCSS = (URL)null;
//		try
//		{
			if(sStyleSheetRef != null && sStyleSheetRef.length() > 0)
			{
				//urlCSS = new URL(this.getCodeBase(), sStyleSheetRef);
			}
//		}
//		catch(MalformedURLException murle)
//		{
//			murle.printStackTrace(System.err);
//		}
		boolean showToolBar = true;
		boolean showToolBarMulti = true;
		//if(this.getParameter("TOOLBAR") != null) { showToolBar = this.getParameter("TOOLBAR").equalsIgnoreCase("true"); }
		//if(this.getParameter("TOOLBARMULTI") != null) { showToolBarMulti = this.getParameter("TOOLBARMULTI").equalsIgnoreCase("true"); 
		//if(showToolBarMulti) { showToolBar = true; }
		//boolean showViewSource = ((this.getParameter("SOURCEVIEW") != null && this.getParameter("SOURCEVIEW").equalsIgnoreCase("true")));
		boolean showViewSource = false;
		//String sLanguage = this.getParameter("LANGCODE");
		String sLanguage = "en";
		//String sCountry = this.getParameter("LANGCOUNTRY");
		String sCountry = "US";
		boolean editModeExclusive = true;
		//if(this.getParameter("EXCLUSIVE") != null) { editModeExclusive = this.getParameter("EXCLUSIVE").equalsIgnoreCase("true"); }
		boolean showMenuIcons = true;
		//if(this.getParameter("MENUICONS") != null) { showMenuIcons = this.getParameter("MENUICONS").equalsIgnoreCase("true"); }
		boolean spellChecker = false;
		//if(this.getParameter("SPELLCHECK") != null) { spellChecker = this.getParameter("SPELLCHECK").equalsIgnoreCase("true"); }
		//String toolbarSeq = (showToolBarMulti ? EkitCore.TOOLBAR_DEFAULT_MULTI : EkitCore.TOOLBAR_DEFAULT_SINGLE);
		//String toolbarSeq = EkitCore.TOOLBAR_DEFAULT_MULTI;
		//System.out.print(EkitCore.TOOLBAR_DEFAULT_MULTI);
		//if(this.getParameter("TOOLBARSEQ") != null) { toolbarSeq = this.getParameter("TOOLBARSEQ").toUpperCase(); }
		String toolbarSeq = "PR|SP|CT|CP|PS|SP|UN|RE|SP|FN|SP|UC|UM|SP|SR|*|BL|IT|UD|SP|SK|SU|SB|SP|AL|AC|AR|AJ|SP|UL|OL|SP|LK|*|ST|SP|FO";
		
		if(spellChecker)
		{
			ekitCore = new EkitCoreSpell(sRawDocument, urlCSS, showToolBar, showViewSource, showMenuIcons, editModeExclusive, sLanguage, sCountry, base64, showToolBarMulti, toolbarSeq);
		}
		else
		{
			ekitCore = new EkitCore(sRawDocument, urlCSS, showToolBar, showViewSource, showMenuIcons, editModeExclusive, sLanguage, sCountry, base64, false, showToolBarMulti, toolbarSeq);
		}

		/* Add menus, based on whether or not they are requested (all are shown by default) */
		Vector<String> vcMenus = new Vector<String>();
		vcMenus.add(EkitCore.KEY_MENU_EDIT); 
		vcMenus.add(EkitCore.KEY_MENU_VIEW); 
		vcMenus.add(EkitCore.KEY_MENU_FONT); 
		vcMenus.add(EkitCore.KEY_MENU_FORMAT); 
		vcMenus.add(EkitCore.KEY_MENU_INSERT); 
		vcMenus.add(EkitCore.KEY_MENU_TABLE); 
		vcMenus.add(EkitCore.KEY_MENU_FORMS); 
		vcMenus.add(EkitCore.KEY_MENU_SEARCH); 
		vcMenus.add(EkitCore.KEY_MENU_TOOLS);
  		//this.getParent().setJMenuBar(ekitCore.getCustomMenuBar(vcMenus));
 

		/* Add the components to the app */

		if(showToolBar)
		{
			if(showToolBarMulti)
			{
				this.setLayout(new GridBagLayout());
				GridBagConstraints gbc = new GridBagConstraints();
				gbc.fill       = GridBagConstraints.HORIZONTAL;
				gbc.anchor     = GridBagConstraints.NORTH;
				gbc.gridheight = 1;
				gbc.gridwidth  = 1;
				gbc.weightx    = 1.0;
				gbc.weighty    = 0.0;
				gbc.gridx      = 1;

				ekitCore.initializeMultiToolbars(toolbarSeq);

				gbc.gridy = 1;
				this.add(ekitCore.getToolBarMain(showToolBar && ekitCore.getToolBarMain(showToolBar).getComponentCount() > 0), gbc);

				gbc.gridy = 2;
				this.add(ekitCore.getToolBarFormat(showToolBar && ekitCore.getToolBarFormat(showToolBar).getComponentCount() > 0), gbc);

				gbc.gridy = 3;
				this.add(ekitCore.getToolBarStyles(showToolBar && ekitCore.getToolBarStyles(showToolBar).getComponentCount() > 0), gbc);

				gbc.anchor     = GridBagConstraints.CENTER;
				gbc.fill       = GridBagConstraints.BOTH;
				gbc.weighty    = 1.0;
				gbc.gridy      = 4;
				this.add(ekitCore, gbc);
			}
			else
			{
				
				ekitCore.initializeSingleToolbar(toolbarSeq);

				this.setLayout(new BorderLayout());
				this.add(ekitCore, BorderLayout.CENTER);
				this.add(ekitCore.getToolBar(showToolBar), BorderLayout.NORTH);
				
			}
		}
		
	}


	/** Method for passing back the document text to the applet's container.
	  * This is the entire document, including the top-level HTML tags.
	  */
	public String getDocumentText()
	{
		return ekitCore.getDocumentText();
	}

	/** Method for passing back the document body to the applet's container.
	  * This is only the text contained within the BODY tags.
	  */
	public String getDocumentBody()
	{
		return ekitCore.getDocumentSubText("body");
	}

	/** Method for setting the document manually.
	  * Will need code in the web page to call this.
	  */
	public void setDocumentText(String text)
	{
		ekitCore.setDocumentText(text);
	}
}
