package WebBuilder;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * Title:        WebpageDB
 * Description:  simple hello world db example of a
 *               standalone persistent db application
 *
 *               every time it runs it adds four more rows to sample_table
 *               it does a query and prints the results to standard out
 *
 * Author: Guy Mann guydmann@gmail.com
 */
public class WebpageDB {
	String DB_NAME;
    Connection conn;                                                //our connnection to the db - presist for life of program

    // we dont want this garbage collected until we are done
    public WebpageDB(String db_file_name_prefix) throws Exception {    // note more general exception
    	DB_NAME =db_file_name_prefix;
        // Load the HSQL Database Engine JDBC driver
        // hsqldb.jar should be in the class path or made part of the current jar
        Class.forName("org.hsqldb.jdbcDriver");

        // connect to the database.   This will load the db files and start the
        // database if it is not alread running.
        // db_file_name_prefix is used to open or create files that hold the state
        // of the db.
        // It can contain directory names relative to the
        // current working directory
        conn = DriverManager.getConnection("jdbc:hsqldb:"
                                           + db_file_name_prefix + "/" + db_file_name_prefix,    // filenames
                                           "sa",                     // username
                                           "");                      // password
    }

    public void shutdown() throws SQLException {

        Statement st = conn.createStatement();

        // db writes out to files and performs clean shuts down
        // otherwise there will be an unclean shutdown
        // when program ends
        st.execute("SHUTDOWN");
        conn.close();    // if there are no other open connection
    }

//use for SQL command SELECT
    public synchronized void query(String expression) throws SQLException {

        Statement st = null;
        ResultSet rs = null;

        st = conn.createStatement();         // statement objects can be reused with

        // repeated calls to execute but we
        // choose to make a new one each time
        rs = st.executeQuery(expression);    // run the query

        // do something with the result set.
        dump(rs);
        st.close();    // NOTE!! if you close a statement the associated ResultSet is

        // closed too
        // so you should copy the contents to some other object.
        // the result set is invalidated also  if you recycle an Statement
        // and try to execute some other query before the result set has been
        // completely examined.
    }

    
    public synchronized Object query_single(String expression) throws SQLException {
    	//will return a single value
    	//should only be used for queries that return a single value
    	
    	
    	Object Return_obj = null;
        Statement st = null;
        ResultSet rs = null;

        st = conn.createStatement();         // statement objects can be reused with

        // repeated calls to execute but we
        // choose to make a new one each time
        rs = st.executeQuery(expression);    // run the query

        // do something with the result set.
        //dump(rs);
        for (; rs.next(); ) {
        	Return_obj = rs.getObject(1);
        }
        
        st.close();    // NOTE!! if you close a statement the associated ResultSet is
        // closed too
        // so you should copy the contents to some other object.
        // the result set is invalidated also  if you recycle an Statement
        // and try to execute some other query before the result set has been
        // completely examined.
        
        return Return_obj;
    }
    
    
//use for SQL commands CREATE, DROP, INSERT and UPDATE
    public synchronized void update(String expression) throws SQLException {

        Statement st = null;

        st = conn.createStatement();    // statements

        int i = st.executeUpdate(expression);    // run the query

        System.out.println( expression);
        if (i == -1) {
            System.out.println("db error : " + expression);
        }

        st.close();
    }    // void update()

    
    public synchronized String[][] query_array(String expression) throws SQLException {

        Statement st = null;
        ResultSet rs = null;
        ResultSet rsTest = null;

        st = conn.createStatement();         // statement objects can be reused with
        rs = st.executeQuery(expression);    // run the query    	
        rsTest = st.executeQuery(expression);    
    	
        // the order of the rows in a cursor
        // are implementation dependent unless you use the SQL ORDER statement
        ResultSetMetaData meta   = rs.getMetaData();
        int               colmax = meta.getColumnCount();
        int               i;
        Object            o = null;
        int rowcheck = 0;
        for (; rsTest.next(); ) {
        	rowcheck++;
        }
        if (rowcheck == 0 ) { return null; } 
        //rs.last();
        int				  rowmax = rowcheck;
        //rs.beforeFirst();
        
		String[][] Return_array = new String[rowmax][colmax] ;
        int count = 0;
                
        // the result set is a cursor into the data.  You can only
        // point to one row at a time
        // assume we are pointing to BEFORE the first row
        // rs.next() points to next row and returns true
        // or false if there is no next row, which breaks the loop
        
//        System.out.print("colmax"+  colmax + "\n");
//        System.out.print("rowmax"+  rowmax + "\n");
        for (; rs.next(); ) {
//            System.out.print("testasdasda" + "\n");
            for (i = 0; i < colmax; i++) {
                o = rs.getObject(i +1);    // Is SQL the first column is indexed
//                System.out.print("test"+  o.toString() + "\n");
                // with 1 not 0
                Return_array[count][i] = o.toString();
            }

            count++;
        }
   
        st.close();
        return Return_array;
    }                                       //void dump( ResultSet rs )

    
    
    public static void dump(ResultSet rs) throws SQLException {

        // the order of the rows in a cursor
        // are implementation dependent unless you use the SQL ORDER statement
        ResultSetMetaData meta   = rs.getMetaData();
        int               colmax = meta.getColumnCount();
        int               i;
        Object            o = null;

        // the result set is a cursor into the data.  You can only
        // point to one row at a time
        // assume we are pointing to BEFORE the first row
        // rs.next() points to next row and returns true
        // or false if there is no next row, which breaks the loop
        for (; rs.next(); ) {
            for (i = 0; i < colmax; ++i) {
                o = rs.getObject(i + 1);    // Is SQL the first column is indexed

                // with 1 not 0
                System.out.print(o.toString() + " ");
            }

            System.out.println(" ");
        }
    }                                       //void dump( ResultSet rs )

    //this will change to be an intialization function but or testing can be a ain function
    public boolean initialize() {


        try {

            //make an empty table
            //
            // by declaring the id column IDENTITY, the db will automatically
            // generate unique values for new rows- useful for row keys
        	
        	//create menu table
            update(
                "CREATE TABLE menu( ID INTEGER IDENTITY, type VARCHAR(10), parent_groupID INTEGER, group_name VARCHAR(25), pageID INTEGER, group_order INTEGER)");
            update(
            	"CREATE TABLE courses_selected( ID INTEGER IDENTITY, courseID INTEGER)");
            update(
            	"CREATE TABLE documents_selected( ID INTEGER IDENTITY, documentID INTEGER)");
                
            
            
            //create page tables
            update(
            	"CREATE TABLE page( ID INTEGER IDENTITY, title VARCHAR(256), name VARCHAR(256), required_page BOOLEAN) ");
            update(
            	"CREATE TABLE page_element( ID INTEGER IDENTITY, element_type VARCHAR(50), elementID INTEGER)");
            update(
            	"CREATE TABLE ref_page_pElement( ID INTEGER IDENTITY, pageID INTEGER, pElementID INTEGER, page_order INTEGER)");
            update(
				"CREATE TABLE image_element( ID INTEGER IDENTITY, title VARCHAR(30), filename VARCHAR(256),alt_text VARCHAR(50), align VARCHAR(20), width INTEGER, height INTEGER)");
            update(
				"CREATE TABLE html_element( ID INTEGER IDENTITY, title VARCHAR(30), html_text VARCHAR(32000))");
            update(
				"CREATE TABLE p_schedule_element( ID INTEGER IDENTITY, schedule_title VARCHAR(30))");
            update(
				"CREATE TABLE sched_event( ID INTEGER IDENTITY, schedID INTEGER, MON BOOLEAN, TUE BOOLEAN, WED BOOLEAN, THU BOOLEAN, FRI BOOLEAN, starttime  VARCHAR(5), endtime  VARCHAR(5), event_text VARCHAR(30))");
            //type should be class or archive
            update(
				"CREATE TABLE applet_element( ID INTEGER IDENTITY, title VARCHAR(30), type VARCHAR(8), height INTEGER, width INTEGER, code VARCHAR(50), src_applet_file_id INTEGER, directory VARCHAR(256))");
            update(
            	"CREATE TABLE applet_files( ID INTEGER IDENTITY, applet_id INTEGER, directory VARCHAR(256), filename VARCHAR(50))");
        
            //create course schedule tables
            update(
        		"CREATE TABLE course( ID INTEGER IDENTITY, title VARCHAR(256), year VARCHAR(2), department VARCHAR(2), term VARCHAR(1), course_number VARCHAR(25), start_date DATE, end_date DATE, lecturer_id INTEGER, coordinator_id INTEGER, lab_coordinator_id INTEGER)");            
            update(
    			"CREATE TABLE section( ID INTEGER IDENTITY, courseID INTEGER, section_number VARCHAR(10), lab_instructor_id1 INTEGER, lab_instructor_id2 INTEGER)");
            update(
				"CREATE TABLE lab( ID INTEGER IDENTITY, courseID INTEGER, title VARCHAR(256))");
            update(
				"CREATE TABLE lab_document( ID INTEGER IDENTITY, lab_id INTEGER, title VARCHAR(256), doc_type VARCHAR(5), external_URL VARCHAR(256), document_id INTEGER)");
            update(
            	"CREATE TABLE ref_section_lab( ID INTEGER IDENTITY, sectionID INTEGER, labID INTEGER, lab_date DATE, lab_time VARCHAR(20))");
        
            update(
            	"CREATE TABLE lecturer( ID INTEGER IDENTITY, lecturer_name VARCHAR(100), lecturer_email VARCHAR(100), lecturer_phone1 VARCHAR(12), lecturer_phone2 VARCHAR(12))");            
            update(
            	"CREATE TABLE coordinator( ID INTEGER IDENTITY, coordinator_name VARCHAR(100), coordinator_email VARCHAR(100), coordinator_phone1 VARCHAR(12), coordinator_phone2 VARCHAR(12))");            
            update(
            	"CREATE TABLE lab_coordinator( ID INTEGER IDENTITY, lab_coordinator_name VARCHAR(100), lab_coordinator_email VARCHAR(100), lab_coordinator_phone1 VARCHAR(12), lab_coordinator_phone2 VARCHAR(12))");               
            update(
            	"CREATE TABLE lab_instructor( ID INTEGER IDENTITY, lab_instructor_name VARCHAR(100), lab_instructor_email VARCHAR(100))");               
            
            
            update(
				"CREATE TABLE document( ID INTEGER IDENTITY, title VARCHAR(256), filename VARCHAR(256), document_group VARCHAR(256), user_doc BOOLEAN)");
            
            
            
        } catch (SQLException ex2) {

            //ignore
            ex2.printStackTrace();  // second time we run program
            //  should throw execption since table
            // already there
            //
            // this will have no effect on the db
            return false;
        }

        try {

            // add some rows - will create duplicates if run more then once
            // the id column is automatically generated
            update(
                "INSERT INTO page(title,name,required_page) VALUES('Home', 'index', True)");
            Object homepageID = null; 
            
            homepageID = query_single("SELECT Max(ID) FROM page");
            
            update(
            	"INSERT INTO page(title,name,required_page) VALUES('Header', 'header', True)");
            update(
            	"INSERT INTO page(title,name,required_page) VALUES('Footer', 'footer', True)");
            update
            	("INSERT INTO menu(type,parent_groupID, group_name,pageID,group_order) VALUES ('page',-1,NULL," + Integer.parseInt(homepageID.toString())   + ",0)");
            
            // at end of program
            //shutdown();
        } catch (SQLException ex3) {
            ex3.printStackTrace();
            return false;
        }
        return true;
    }    // initialize 
    
    //this will change to be an intialization function but or testing can be a ain function
    public static void main(String[] args) {

        WebpageDB db = null;

        try {
            db = new WebpageDB("db_file");
        } catch (Exception ex1) {
            ex1.printStackTrace();    // could not start db

            return;                   // bye bye
        }
        db.initialize();
    }    // main()
}    // class Testdb
